﻿-- Author      : RisM
-- Create Date : 9/21/2009 3:06:34 AM

local SpeakinSpell = LibStub("AceAddon-3.0"):GetAddon("SpeakinSpell")
local L				= LibStub("AceLocale-3.0"):GetLocale("SpeakinSpell",					false)
local BIGREDBUTTON	= LibStub("AceLocale-3.0"):GetLocale("SpeakinSpell_BIGREDBUTTON",		false)

SpeakinSpell:PrintLoading("gui/generaloptions.lua")


-------------------------------------------------------------------------------
-- GUI LAYOUT - GENERAL SETTINGS
-------------------------------------------------------------------------------


SpeakinSpell.OptionsGUI.args.General = {
	order = 1,
	type = "group",
	name = L["General Settings"],
	desc = L["General Settings TOOLTIP"],
	args = {
		Caption = {
			order = 1,
			type = "description",
			name = L["General Settings"],
		},
		ShowVersion = {
			order = 2,
			type = "description",
			name = L["Version "]..SpeakinSpell.CURRENT_VERSION,
		},
		ShowMemory = {
			order = 3,
			type = "description",
			name = function() return SpeakinSpell:GetAddonMemoryUsedString() end,
		},
		ShowURL = {
			order = 4,
			type = "description",
			name = SpeakinSpell.URL,
		},
		EnableAllMessagesToggle = {
			order = 5,
			type = "toggle",
			width = "full",
			name = L["Enable All Messages"],
			desc = L["Enable All Messages TOOLTIP"],
			get = function() return  SpeakinSpell:GeneralOptions_OnEnableAllMessagesToggle("GET",nil) end,
			set = function(_, value) SpeakinSpell:GeneralOptions_OnEnableAllMessagesToggle("SET",value) end,
		},
		EnableMinimapIcon = {
			order = 5,
			type = "toggle",
			name = L["Show Minimap Button"],
			desc = L["Show the SpeakinSpell minimap button"],
			get = function(info) return SpeakinSpellSavedData.ShowMinimapButton end,
			set = function(info, value) SpeakinSpell:MinimapButton_Show(value) end,
		},
		ShowSpellEventsToggle = {
			order = 6,
			type = "toggle",
			width = "full",
			name = L["Report detected spell events"],
			desc = L["Report detected spell events TOOLTIP"],
			get = function() return  SpeakinSpell:GeneralOptions_OnShowSpellEventsToggle("GET",nil) end,
			set = function(_, value) SpeakinSpell:GeneralOptions_OnShowSpellEventsToggle("SET",value) end,
		},
		ShowDebugMessagesToggle = {
			order = 7,
			type = "toggle",
			width = "full",
			name = L["Show Debugging Messages (verbose)"],
			desc = L["Show Debugging Messages TOOLTIP"],
			get = function() return  SpeakinSpell:GeneralOptions_OnShowDebugMessagesToggle("GET",nil) end,
			set = function(_, value) SpeakinSpell:GeneralOptions_OnShowDebugMessagesToggle("SET",value) end,
		},
		ImportDefaultsButton = {
			order = 8,
			type = "execute",
			width = "full",
			name = L["Import New Default Sample Speeches"],
			desc = L["Import New Default Sample Speeches TOOLTIP"],
			func = function() SpeakinSpell:GeneralOptions_OnClickImport() end,
		},
		ScrollGap = {
			order = 9,
			type = "description",
			name = "\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n",
			func = function() SpeakinSpell:GeneralOptions_OnClickImport() end,
		},
		BigRedButton = {
			order = 10,
			type = "execute",
			width = "full",
			name = L["DO NOT PRESS THIS BUTTON"],
			desc = L["Under no circumstances should you ever, EVER push this button.\n\nI'm warning you.\n\nDon't do it.\n\nSeriously, you're already hovering over it for too long, and that's dangerous.\n\nMove your mouse before it's too late!"],
			func = function() SpeakinSpell:GeneralOptions_OnClickBigRedButton() end,
		},
	}, --end args
}

-------------------------------------------------------------------------------
-- OPTIONS GUI FUNCTIONS - GENERAL SETTINGS
-------------------------------------------------------------------------------


function SpeakinSpell:GeneralOptions_OnShowSpellEventsToggle(getset, value)
	if "GET" == getset then
		return SpeakinSpellSavedData.ShowSpellEvents
	else -- "SET"
		SpeakinSpellSavedData.ShowSpellEvents = value
	end
end


function SpeakinSpell:GeneralOptions_OnShowDebugMessagesToggle(getset, value)
	if "GET" == getset then
		return SpeakinSpellSavedData.ShowDebugMessages
	else -- "SET"
		SpeakinSpellSavedData.ShowDebugMessages = value
	end
end


function SpeakinSpell:GeneralOptions_OnEnableAllMessagesToggle(getset, value)
	if "GET" == getset then
		return SpeakinSpellSavedData.EnableAllMessages
	else -- "SET"
		SpeakinSpellSavedData.EnableAllMessages = value
	end
end


function SpeakinSpell:GeneralOptions_OnClickImport()
	self:ImportNewDefaults()
end


function SpeakinSpell:GeneralOptions_OnClickBigRedButton()
	-- check range of BigRedButtonIndex
	if not self.RuntimeData.BigRedButtonIndex then
		self.RuntimeData.BigRedButtonIndex = 1
	end
	if self.RuntimeData.BigRedButtonIndex > #(BIGREDBUTTON.List) then
		self.RuntimeData.BigRedButtonIndex = 1
	end
	
	-- process substitution variables
	local DetectedEventStub = {
		name = "Big Red Button",
		rank = "Silly",
		target = self:GetDefaultTarget(false),
		caster = UnitName("player"),
		type = "MACRO",
		linenumber = tostring(self.RuntimeData.BigRedButtonIndex),
	}
	local DetectedEvent = self:CreateDetectedEvent( DetectedEventStub )
	local msg = BIGREDBUTTON.List[ self.RuntimeData.BigRedButtonIndex ]
	
	-- print the next message in the looping array
	if msg ~= "" then --NOTE: some empty items are inserted on purpose to make a click non-functional
		self:SayOneLine( msg, "SPEAKINSPELL CHANNEL", nil, DetectedEvent )
	end
	
	-- prepare for the next pass
	self.RuntimeData.BigRedButtonIndex = self.RuntimeData.BigRedButtonIndex + 1
end
