﻿-- Author      : RisM
-- Create Date : 6/28/2009 4:04:27 PM

local SpeakinSpell = LibStub("AceAddon-3.0"):GetAddon("SpeakinSpell")
local L	= LibStub("AceLocale-3.0"):GetLocale("SpeakinSpell", false)

SpeakinSpell:PrintLoading("speech.lua")

-------------------------------------------------------------------------------
-- RANDOM SPEECHES - OnSpeechEvent, Speak or execute a command
-------------------------------------------------------------------------------


function SpeakinSpell:OnSpeechEvent( DetectedEventStub )
	local funcname = "OnSpeechEvent"

	-- fill in the stub
	local DetectedEvent = self:CreateDetectedEvent( DetectedEventStub )
	
	local rankedkey, ranklesskey = self:MakeEventTableKeys( DetectedEvent )
	
	local SpeakForRankless = true
	
	--NOTE: it looks like I want to make a subroutine here and call it twice
	--		but the logic is actually slightly different between the ranked and rankless cases
	
	-- look for an event table entry for the ranked version
	if rankedkey then
		local de = self:CopyTable( DetectedEvent )
		de.key = rankedkey
		de.EventTableEntry = SpeakinSpellSavedData.EventTable[ de.key ]
		
		if SpeakinSpellSavedData.ShowSpellEvents then
			self:Print( self:FormatSubs( "<EventTypePrefix><SpellLink>", de) )
		end

		if de.EventTableEntry then
			SpeakForRankless = false
			if self:AllowSpeakForSpell( de ) then
				self:SpeakForSpell( de )
			end
		else
			-- record a new event for this ranked key
			--self:DebugMsg(funcname, "no EventTableEntry for key:"..tostring(de.key))
			self:RecordNewEvent( de ) -- this updates the Create New window as necessary			
		end
	end
	
	-- check for a rankless version, but don't speak for both
	if ranklesskey then
		local de = self:CopyTable( DetectedEvent )
		de.key = ranklesskey
		de.EventTableEntry = SpeakinSpellSavedData.EventTable[ de.key ]

		if SpeakinSpellSavedData.EventTable[ de.key ] then
			if SpeakForRankless then
				if self:AllowSpeakForSpell( de ) then
					self:SpeakForSpell( de )
				end
			end
		else
			-- record a new RANKLESS event for this rankless key
			de.rank = "" -- to make the new event rankless -- NOTE: don't erase this before speaking, to maintain validity of <spellrank> variable
			de.displayname = self:FormatDisplayName( de ) -- update display name to show "any rank"
			--self:DebugMsg(funcname, "no EventTableEntry for key:"..tostring(de.key))
			self:RecordNewEvent( de ) -- this updates the Create New window as necessary
		end
	end

end



function SpeakinSpell:OnRecursiveEventDetected( text, DetectedEventStub, ParentDetectedEvent )
	local funcname = "OnSpeechEvent"

	-- fill in the stub
	local DetectedEvent = self:CreateDetectedEvent( DetectedEventStub )
	-- note: recursive events only have a ranked key
	local key, _ = self:MakeEventTableKeys( DetectedEvent )
	
	if not key then
		self:DebugMsg(funcname,"ERROR: undefined key")
		return
	end
	--self:DebugMsg(funcname,"key:"..key)

	-- look for an event table entry for the ranked version
	local de = self:CopyTable( DetectedEvent )
	de.key = key
	de.EventTableEntry = SpeakinSpellSavedData.EventTable[ de.key ]
	
	if SpeakinSpellSavedData.ShowSpellEvents then
		self:Print( self:FormatSubs( "<EventTypePrefix><SpellLink>", de) )
	end

	if not de.EventTableEntry then
		self:Print("ERROR: no event settings for "..text)
		return
	end

	if not self:AllowSpeakForSpell( de ) then
		return
	end
	
	local pop = true
	if not self.RuntimeData.ParentDetectedEvent then
		self.RuntimeData.ParentDetectedEvent = ParentDetectedEvent
	else
		-- add the new cascading event info the old one, in case it defines anything new
		self:ValidateObject( self.RuntimeData.ParentDetectedEvent, ParentDetectedEvent )
		pop = false
	end

	self:SpeakForSpell( de )

	if pop then
		self.RuntimeData.ParentDetectedEvent = nil	
	end
end


function SpeakinSpell:GetChatChannelForSpell(EventTableEntry)
	local funcname = "GetChatChannelForSpell"
	
	local key = self:GetScenarioKey()
	local channel = EventTableEntry.Channels[key]
	
	-- if the player wants to speak in the RAID_WARNING channel, 
	-- but doesn't have permission (not the leader or assist)
	-- then fall back to the RAID channel instead, so we say something at all
	-- instead of triggering an error message in the chat
	if (channel == "RAID_WARNING") and 
		not (IsRaidOfficer() or IsRaidLeader() or IsPartyLeader()) then
		self:DebugMsg(funcname, "You don't have permission to say Raid Warnings right now - using RAID chat instead")
		channel = "RAID"
	end
	
	return channel
end



-- return true if this is a slash command
-- return false if this is other default text
function SpeakinSpell:ProcessChatSlashCommand(text, DetectedEvent)
	local funcname = "ProcessChatSlashCommand"
	
	if ( strsub(text, 1, 1) ~= "/" ) then
		return false
	end
	
	-- issue a warning if this a secure command like /cast
	-- If the string is in the format "/cmd blah", command will be "/cmd"
	local command = strmatch(text, "^(/[^%s]+)") or "";
	if IsSecureCmd(command) then
		self:Print(string.format(L["WARNING: can't execute protected command: %s"],text))
		return true
	end
	
	-- if this is a recursive call into a slash command, it needs some special handling
	-- do it with internal calls instead of passing through the global chat parser and back again
	if self:ProcessRecursiveUserMacro(text, DetectedEvent) then
		return true
	end
	
	self:DebugMsg(funcname,"executing slash command: " .. text)
	
	-- create a hidden edit box to parse and send the slash command
	if not SpeakinSpellTempEditBox then
		SpeakinSpellTempEditBox = CreateFrame("EditBox", "SpeakinSpellTempEditBox", UIParent)
		SpeakinSpellTempEditBox:Hide()
	end
	
	-- send the command
	SpeakinSpellTempEditBox:SetText(text) 
	ChatEdit_SendText(SpeakinSpellTempEditBox)

	return true
end



function SpeakinSpell:SplitLines(msg)
	local s1 = string.gsub(msg, "<newline>", "\n")
	local s2 = string.gsub(s1, L["<newline>"], "\n")
	local t = { strsplit("\n",s2) }
	return t
end


function SpeakinSpell:SayOneLine(line, channel, WhisperTarget, DetectedEvent)
	line = self:FormatSubs(line, DetectedEvent)

	-- if this message is a slash command, send it through an editBox function to parse and execute the command
	--NOTE: a specified slash command overrides the default channel and whisper features below
	--		it doesn't make sense to combine them
	if self:ProcessChatSlashCommand(line, DetectedEvent) then
		return
	end
	
	-- say the message to the chat, if applicable
	if channel then
		if channel == "SPEAKINSPELL CHANNEL" then
			self:Print(line)
		else
			local language = nil --TODOFUTURE: allow for other languages
			--TODOFUTURE: should this check for "|" for invalid escape sequences to prevent LUA errors? or allow color changes and stuff?
			SendChatMessage(line, channel, language);
		end
	end
	
	-- send the message to a whisper to the target, if applicable
	if WhisperTarget then
		-- NOTE: always use Common(nil) for the language just to ensure the person you whisper can read what you say
		SendChatMessage(line, "WHISPER", nil, DetectedEvent.target);
	end
end


function SpeakinSpell:SpeakForSpell(DetectedEvent)
	local funcname = "SpeakForSpell"
	
	-- determine which channel we will speak in
	channel = self:GetChatChannelForSpell(DetectedEvent.EventTableEntry)
	
	--TODOFUTURE: make it an option to whisper yourself - but be wary of the auto self-cast logic
	local WhisperTarget = DetectedEvent.EventTableEntry.WhisperTarget and DetectedEvent.target and (DetectedEvent.target ~= "") and (DetectedEvent.target ~= "Unknown") and (DetectedEvent.target ~= UnitName("player")) and UnitIsFriend("player",DetectedEvent.target)

	if not channel and not WhisperTarget then
		-- this spell's speeches have no target for the current situation
		-- NOTE: this rule applies even if a slash command override is present in the randomly selected message text
		self:DebugMsg(funcname, self:FormatSubs("<SpellLink> is silent while <Scenario>",DetectedEvent) )
		return
	end
	
	-- select a random speech
	local msg = self:GetRandomTableEntry( DetectedEvent.EventTableEntry.Messages, self:GetLastMessage( DetectedEvent ) )
	if not msg then
		self:DebugMsg(funcname, self:FormatSubs("<SpellLink> doesn't have any speeches defined",DetectedEvent) )
		return
	end
	
	-- record the time when we spoke for this spell, along with the last target, and last message spoken
	-- to support duplicate prevent, announcement limits, and cooldowns
	-- NOTE: record the unprocessed message text, so that changing the value of substitution variables
	--		does not make us think it's a new random message selection
	self:SaveAnnouncementHistory( DetectedEvent, msg )
	
	-- break up the message into separate lines, if applicable
	local lines = self:SplitLines(msg)

	for _,line in ipairs(lines) do
		-- process substitution variables
		self:SayOneLine(line, channel, WhisperTarget, DetectedEvent)
	end
end

